/* Name: 1-wire.c
 * Project: Comfort_termometer
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2011-01-23
 * Tabsize: 4
 * Copyright: (c) 2011 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Comfort_termometer.
 *
 *   Comfort_termometer is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Comfort_termometer is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Comfort_termometer.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/* 
* timer 2 set on 500kHz clk 
* v. 23-01-2011
*/
#include <compat/deprecated.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include  <util/crc16.h>

#include "main.h"
#include "1-wire.h"

//==================================================================
//1wire atmega DRIVERS
//
//-------------------------------------------------------------------------
//This procedure calculates the cumulative Dallas Semiconductor 1Wire CRC of all bytes passed to it. The result
//accumulates in the global variable CRC.
uint8_t CRC8(uint8_t inData, uint8_t seed)
{//from AVR318
volatile    uint8_t bitsLeft;
volatile    uint8_t temp;

    for (bitsLeft = 8; bitsLeft > 0; bitsLeft--)
    {
        temp = ((seed& 0x01) ^ (inData & 0x01));
        if (temp == 0)
        {
            seed >>= 1;
        }
        else
        {
            seed ^= 0x18;
		  // seed ^=0x99;
            seed >>= 1;
            seed |= 0x80;
        }
        inData >>= 1;
    }
//	seed=2;
    return seed;
}


void strongPullUp1sec(void)
	{
	uint16_t i;
	OUT_1_WIRE_HIGH;
	
	for (i=0;i<600;i++)
		{
		sbi(TIFR,OCF1A);
		TCNT1=0;
		OCR1A=600;
		
		
		TCCR1B=_BV(CS11);
		while (bit_is_clear(TIFR,OCF1A));
		}
	
	OUT_1_WIRE_DISCONNECT;
	}


uint8_t startInitialization (void)
{

uint8_t error=1;
sbi(TIFR,OCF1A);
TCNT1=0;
OCR1A=260;
cli();
OUT_1_WIRE_LOW;
TCCR1B=_BV(CS11);

while (bit_is_clear(TIFR,OCF1A));
TCCR1B=0;
OUT_1_WIRE_DISCONNECT;
sbi(TIFR,OCF1A);
TCNT1=0;
OCR1A=260;
TCCR1B=_BV(CS11);
while (bit_is_clear(TIFR,OCF1A))
	{
	if ((TCNT1>30)&&(bit_is_clear(PINB,3))) error=0;
	}
	sei();
TCCR1B=0;
return error;

}



void writeByte (uint8_t byte)
{
uint8_t j1=1,i;
cli();
for (i=0;i<8;i++)
	{
	
	OUT_1_WIRE_LOW;
	if (byte&j1) 
		{
		sbi(TIFR,OCF1A);
		OCR1A=1;
		TCNT1=0;
		TCCR1B=_BV(CS11);
		while (bit_is_clear(TIFR,OCF1A));
		TCCR1B=0;
		OUT_1_WIRE_DISCONNECT;
		OCR1A=18;
		}
	else OCR1A=20;
	sbi(TIFR,OCF1A);
	TCNT1=0;
	TCCR1B=_BV(CS11);
	while (bit_is_clear(TIFR,OCF1A));
	
	OUT_1_WIRE_DISCONNECT;
	
	if (i==7&&byte==0x44) 
		{
		OUT_1_WIRE_HIGH;
		
		}
	sbi(TIFR,OCF1A);
	OCR1A=20;
	TCNT1=0;
	TCCR1B=_BV(CS11);
	while (bit_is_clear(TIFR,OCF1A));
	TCCR1B=0;
	j1<<=1;
		
	}
	sei();


}

uint8_t readByte (void)
{
	uint8_t byte=0,i;
cli();
	for(i=0;i<8;i++)
	{
	
	byte>>=1;
	OUT_1_WIRE_LOW;
	sbi(TIFR,OCF1A);
		OCR1A=1;
		TCNT1=0;
		TCCR1B=_BV(CS11);
		while (bit_is_clear(TIFR,OCF1A));
		TCCR1B=0;
	OUT_1_WIRE_DISCONNECT;
	sbi(TIFR,OCF1A);
		OCR1A=4;
		TCNT1=0;
		TCCR1B=_BV(CS11);
		while (bit_is_clear(TIFR,OCF1A));
	if (bit_is_set(PINB,3)) byte|=0x80;
	TCCR1B=0;
	
	OCR1A=33;
	sbi(TIFR,OCF1A);
	TCNT1=0;
	TCCR1B=_BV(CS11);
	while (bit_is_clear(TIFR,OCF1A));
	TCCR1B=0;
	
	}
	sei();


return byte;
}

uint16_t readThemperature (void)
{
uint16_t temperature=0x07D0;
static volatile uint8_t readArray[9],*p_rArray,Crc,i;

readArray[8]=0;

if (startInitialization()==0)
	{
	writeByte(0xCC);
	writeByte(0x44);
	strongPullUp1sec();
	
	
		{
		Crc=0;
		p_rArray=readArray;
		startInitialization();
		writeByte(0xCC);
		writeByte(0xBE);
	
		
		for (i=0;i<9;i++)
			{
			*p_rArray=readByte();
			if (i<8) Crc=CRC8(*p_rArray, Crc);
		//	Crc=_crc_ibutton_update(Crc,*p_rArray);
			p_rArray++;
			
			}
		}
	if (readArray[8]==Crc) temperature=0x100*readArray[1]+readArray[0];
	}
	
	
//	temperature=readArray[8];
//	temperature=Crc;
	return temperature;
}