/* Name: USI.c
 * Project: Comfort_termometer
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2011-03-26
 * Tabsize: 4
 * Copyright: (c) 2011 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Comfort_termometer.
 *
 *   Comfort_termometer is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Comfort_termometer is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Comfort_termometer.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */




/* transmit data to indicators via USI

*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <compat/deprecated.h>

#include "main.h"
#include "USI.h"

void USIinit(void)
{

UCSRC=_BV(URSEL)|_BV(UMSEL)|_BV(UPM1)|_BV(UCSZ1)|_BV(UCSZ0);

UBRRL=1;
UCSRB=_BV(TXEN)|_BV(UCSZ2);
}

void USIDeInit(void)
{
UCSRB=0;
}

/*

in DataBlock 4 bytes - indicating value, in 5 byte high half-bytes is point position, low half-bytes is bright level
*/
void transmitDataStructure(struct dataIndic *aData)
{
uint8_t DataBlock[5],*p_Datablock;
uint8_t a;
uint16_t abs;
//uint16_t b;

p_Datablock=DataBlock;
if (aData->AirTemperature<0)
	{
	 *p_Datablock=11;
	 abs=-aData->AirTemperature;
	 }
else 
	{
	*p_Datablock=10;
	abs=aData->AirTemperature;
	}

p_Datablock++;
*p_Datablock=abs/100;
p_Datablock++;
a=abs%100;
*p_Datablock=a/10;
p_Datablock++;
*p_Datablock=a%10;
p_Datablock++;
*p_Datablock=0x20|aData->bright;
transmitDataBlock(1,DataBlock);

p_Datablock=DataBlock;
if (aData->comfortTemp<0)
	{
	 *p_Datablock=11;
	 abs=-aData->comfortTemp;
	 }
else 
	{

	*p_Datablock=10;
	abs=aData->comfortTemp;
	}
p_Datablock++;
*p_Datablock=abs/100;
p_Datablock++;
a=abs%100;
*p_Datablock=a/10;
p_Datablock++;
*p_Datablock=a%10;


p_Datablock++;

*p_Datablock=0x20|(aData->bright/2-aData->bright/4);

transmitDataBlock(2,DataBlock);


p_Datablock=DataBlock;
*p_Datablock=aData->hour/10;
p_Datablock++;
*p_Datablock=aData->hour%10;
p_Datablock++;
*p_Datablock=aData->minutes/10;
p_Datablock++;
*p_Datablock=aData->minutes%10;
p_Datablock++;
if (aData->clockPoint) *p_Datablock=0x30|aData->bright;
else *p_Datablock=aData->bright;

transmitDataBlock(3,DataBlock);
}



void transmitDataBlock(uint8_t aAdress,uint8_t* p_aDataBlock)
{
uint8_t i;
volatile uint8_t j;
transmitAdress(aAdress);

for (i=0;i<5;i++)
	{
	for (j=0;j<10;j++);
	
	transmitByte(*p_aDataBlock);

	p_aDataBlock++;
	
	}
}

void transmitByte(uint8_t aByte)
{
uint8_t n;
volatile uint8_t i;

for (i=0;i<5;i++);
cbi (UCSRB,TXB8);

if (UCSRA & (1<<RXC)) n=UDR;
UDR=aByte;
while ( !( UCSRA & (1<<UDRE)) );

}

void transmitAdress(uint8_t aByte)
{
uint8_t n;
sbi (UCSRB,TXB8);
if (UCSRA & (1<<RXC)) n=UDR;
UDR=aByte;
while ( !( UCSRA & (1<<UDRE)) );

}