/* Name: main.c
 * Project: Comfort_termometer
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2011-03-30
 * Tabsize: 4
 * Copyright: (c) 2011 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Comfort_termometer.
 *
 *   Comfort_termometer is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Comfort_termometer is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Comfort_termometer.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */



/* control program for clock-thermometer-windchill thermometer
* 3 outer indicators connected over USI,
* for counting time using timer 2, 1-wire use timer1, timer 0 used for const power regulator,
* determination power-off state and switch to energy saving mode
* measure air temperature every 10 min, wind chill temperature every 30min
*/

#include <compat/deprecated.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include  <avr/sleep.h>
#include <avr/wdt.h>

#include "main.h"
#include "iohardware.h"
#include "1-wire.h"
#include "termo_measure.h"
#include "USI.h"
#include "clock_tick.h"
#include "base_measure.h"

volatile uint32_t clocks __attribute__  ((section  (".noinit")));

void watchDogReset (void)
{
if (timer2CircleFlag&&ADCcircleFlag)
		{
		timer2CircleFlag=0;
		ADCcircleFlag=0;
		if (VoltageStabilizerFlag==ON)
			{
			if (timer0CircleFlag) 
				{
				timer0CircleFlag=0;
				wdt_reset();
				}
			}
		else wdt_reset();
		}
}



int main (void)
{

// set pointers to independent memory cells
p_clockticks=0xae;
p_checkSum=0xba;

clocks=*p_clockticks;

uint8_t check;

check=clocks;
check^=clocks>>8;
check^=clocks>>16;
check^=clocks>>24;
if (*p_checkSum==check) clockTicks=clocks;


iohardwareInit();
data.bright=0x8;
data.AirTemperature=999;
data.comfortTemp=999;

/* the problem of start calibration this is a periodic damaging of eeprom cells value (one in few monthes), because
* for robust work device make next sequence:
* uncomment line Kstatic=base_measure() and build program, write program to controller,
* power on circuit in indoor condition and wait about 10 min, until calibrate go. Read
* overheat value from wind chill temperature indicator, again comment line Kstatic=base_measure(), 
* write overheat value without point to line Kstatic= overheat value (overheat of my sensor is 28.8 Celsium degrees), after all build 
* program and write for your device.
*
*/

//Kstatic=base_measure();
Kstatic=288;

for (;;)
	{
	sei();

	if (VoltageShutdown)
		{
		iohardwareDeInit();
		
		set_sleep_mode(SLEEP_MODE_ADC);
		sei();
		sleep_mode();
		iohardwareInit();
		VoltageShutdown=0;
		}
	else 
		{
		
		
		
			{
				
				static int16_t comfortTemp=999,AirTemperature=999;
				
				if (measureAirTemperatureFlag)
					{
					
					 measureTemperature (&comfortTemp,&AirTemperature);
					 measureAirTemperatureFlag=0;
					
			
					if (VoltageStabilizerFlag==ON) TCCR0=_BV(CS01);
					else 
						{
						TCCR0=0;
						SWITCH_OFF;
						}
					cli();
					
					data.AirTemperature=AirTemperature;
					data.comfortTemp=comfortTemp;
					sei();
					}
			}
		
		}
	watchDogReset();
	
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_mode();
	}
return (0);
}