/* Name: termo_measure.c
 * Project: Comfort_termometer
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2011-03-26
 * Tabsize: 4
 * Copyright: (c) 2011 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Comfort_termometer.
 *
 *   Comfort_termometer is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Comfort_termometer is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Comfort_termometer.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


/*
process temperature measurement and calculation. sensor DS18B20
*/

#include <compat/deprecated.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "main.h"
#include "1-wire.h"
#include "termo_measure.h"

int16_t readFilteredTemperature()
{
static int16_t FilterValue=100; //-55 Celsium degreese
static int16_t ValueStep1=1,ValueStep2=2,ValueStep3=3;

do
	{
	uint8_t i;
	for (i=0;i<2;i++)
	{
	watchDogReset();
	FilterValue=readThemperature()+FilterValue-FilterValue/2;
	}
	ValueStep3=ValueStep2;
	ValueStep2=ValueStep1;
	ValueStep1=FilterValue/3.2; // Celsium scale
//	ValueStep1==FilterValue/1.778+32; // Fahrenheit scale
	
	}
while ((ValueStep1!=ValueStep2)||(ValueStep2!=ValueStep3));


return ValueStep1;

}

int16_t calculateComfortTemperature(int16_t AirTemperature,int16_t HotSensorTemp)
{
/* baseComfortTemperature is a temperature of human body.
* in Celsium scale this is 366, in Fahrenheit 982
* Ksensor is sensor correction coefficient, with this coefficient this formula give 
* a plausible value of wind chill temperature
*/
const int16_t baseComfortTemperature=366,Ksensor=4;
int16_t comfortTemp;

float Kkorrekt;
Kkorrekt=((float)Kstatic/(HotSensorTemp-AirTemperature))/Ksensor+(1-1/(float)Ksensor);

if (AirTemperature<baseComfortTemperature)
	{
	comfortTemp=baseComfortTemperature-Kkorrekt*(baseComfortTemperature-AirTemperature);
	
	}

else 
	{
	comfortTemp=AirTemperature-(Kkorrekt-1)*(AirTemperature-baseComfortTemperature);
	
	}

return comfortTemp;

}




void measureTemperature (int16_t* p_comfortTemp,int16_t* p_AirTemperature)
{

static uint8_t idleCircle;

if (measureComfortTemperatureFlag)
	{
	// power on sensor heater and wait 10 min, until next function call
	if (VoltageStabilizerFlag==ON)
		{
		*p_comfortTemp=readFilteredTemperature();
		measureComfortTemperatureFlag=0;
		idleCircle=1;
		VoltageStabilizerFlag=OFF;
		*p_comfortTemp=calculateComfortTemperature(*p_AirTemperature,*p_comfortTemp);
		}
	else 
		{
		*p_AirTemperature=readFilteredTemperature();
		VoltageStabilizerFlag=ON;
		}
	
	}
else 
	{
	
	if (idleCircle) idleCircle=0;
	else *p_AirTemperature=readFilteredTemperature();
	}

}
